@@ -7,11 +7,23 @@ use core::any::{Any, TypeId};
 
 // yolo
 
+/// Get a `type_id` from it's previously unpacked `u64`.
+/// Opposite of [`unpack_type_id(id)`].
+///
+/// # Safety
+/// Probably not safe for future compilers, fine for now.
+#[must_use]
 pub fn pack_type_id(id: u64) -> TypeId {
     assert_eq_size!(TypeId, u64);
     unsafe { *(&id as *const u64 as *const TypeId) }
 }
 
+/// Unpack a `type_id` to an `u64`
+/// Opposite of [`pack_type_id(id)`].
+///
+/// # Safety
+/// Probably not safe for future compilers, fine for now.
+#[must_use]
 pub fn unpack_type_id(id: TypeId) -> u64 {
     assert_eq_size!(TypeId, u64);
     unsafe { *(&id as *const _ as *const u64) }
@@ -25,11 +37,13 @@ pub trait SerdeAny: Any + erased_serde::Serialize {
     fn as_any_mut(&mut self) -> &mut dyn Any;
 }
 
+/// Wrap a type for serialization
 pub struct Wrap<'a, T: ?Sized>(pub &'a T);
 impl<'a, T> Serialize for Wrap<'a, T>
 where
     T: ?Sized + erased_serde::Serialize + 'a,
 {
+    /// Serialize the type
     fn serialize<S>(&self, serializer: S) -> Result<S::Ok, S::Error>
     where
         S: Serializer,
@@ -38,13 +52,16 @@ where
     }
 }
 
+/// Callback for [`SerdeAny`] deserialization.
 pub type DeserializeCallback<B> =
     fn(&mut dyn erased_serde::Deserializer) -> Result<Box<B>, erased_serde::Error>;
 
+/// Callback struct for deserialization of a [`SerdeAny`] type.
 pub struct DeserializeCallbackSeed<B>
 where
     B: ?Sized,
 {
+    /// Callback for deserialization of a [`SerdeAny`] type.
     pub cb: DeserializeCallback<B>,
 }
 
@@ -58,14 +75,17 @@ where
     where
         D: serde::de::Deserializer<'de>,
     {
-        let mut erased = erased_serde::Deserializer::erase(deserializer);
+        let mut erased = <dyn erased_serde::Deserializer>::erase(deserializer);
         (self.cb)(&mut erased).map_err(serde::de::Error::custom)
     }
 }
 
+/// Creates the [`serde`] registry for serialization and deserialization of [`SerdeAny`].
+/// Each element needs to be registered so that it can be deserialized.
 #[macro_export]
 macro_rules! create_serde_registry_for_trait {
     ($mod_name:ident, $trait_name:path) => {
+        /// A [`crate::bolts::serdeany`] module.
         pub mod $mod_name {
 
             use alloc::boxed::Box;
@@ -82,6 +102,7 @@ macro_rules! create_serde_registry_for_trait {
             };
             use $crate::Error;
 
+            /// Visitor object used internally for the [`SerdeAny`] registry.
             pub struct BoxDynVisitor {}
             impl<'de> serde::de::Visitor<'de> for BoxDynVisitor {
                 type Value = Box<dyn $trait_name>;
@@ -194,6 +215,8 @@ macro_rules! create_serde_registry_for_trait {
             }
 
             impl SerdeAnyMap {
+                /// Get an element from the map.
+                #[must_use]
                 #[inline]
                 pub fn get<T>(&self) -> Option<&T>
                 where
@@ -204,6 +227,8 @@ macro_rules! create_serde_registry_for_trait {
                         .map(|x| x.as_ref().as_any().downcast_ref::<T>().unwrap())
                 }
 
+                /// Get a mutable borrow for an element in the map.
+                #[must_use]
                 #[inline]
                 pub fn get_mut<T>(&mut self) -> Option<&mut T>
                 where
@@ -214,6 +239,7 @@ macro_rules! create_serde_registry_for_trait {
                         .map(|x| x.as_mut().as_any_mut().downcast_mut::<T>().unwrap())
                 }
 
+                /// Insert an element into the map.
                 #[inline]
                 pub fn insert<T>(&mut self, t: T)
                 where
@@ -223,11 +249,15 @@ macro_rules! create_serde_registry_for_trait {
                         .insert(unpack_type_id(TypeId::of::<T>()), Box::new(t));
                 }
 
+                /// Returns the count of elements in this map.
+                #[must_use]
                 #[inline]
                 pub fn len(&self) -> usize {
                     self.map.len()
                 }
 
+                /// Returns if the map contains the given type.
+                #[must_use]
                 #[inline]
                 pub fn contains<T>(&self) -> bool
                 where
@@ -236,6 +266,8 @@ macro_rules! create_serde_registry_for_trait {
                     self.map.contains_key(&unpack_type_id(TypeId::of::<T>()))
                 }
 
+                /// Create a new [`SerdeAnyMap`].
+                #[must_use]
                 pub fn new() -> Self {
                     SerdeAnyMap {
                         map: HashMap::default(),
@@ -249,12 +281,15 @@ macro_rules! create_serde_registry_for_trait {
                 }
             }
 
+            /// A serializable [`HashMap`] wrapper for [`SerdeAny`] types, addressable by name.
             #[derive(Serialize, Deserialize)]
             pub struct NamedSerdeAnyMap {
                 map: HashMap<u64, HashMap<u64, Box<dyn $trait_name>>>,
             }
 
             impl NamedSerdeAnyMap {
+                /// Get an element by name
+                #[must_use]
                 #[inline]
                 pub fn get<T>(&self, name: &str) -> Option<&T>
                 where
@@ -268,6 +303,8 @@ macro_rules! create_serde_registry_for_trait {
                     }
                 }
 
+                /// Get an element of a given type contained in this map by [`TypeId`].
+                #[must_use]
                 #[inline]
                 pub fn by_typeid(&self, name: &str, typeid: &TypeId) -> Option<&dyn $trait_name> {
                     match self.map.get(&unpack_type_id(*typeid)) {
@@ -278,6 +315,8 @@ macro_rules! create_serde_registry_for_trait {
                     }
                 }
 
+                /// Get an element of a given type contained in this map by [`TypeId`], as mut.
+                #[must_use]
                 #[inline]
                 pub fn get_mut<T>(&mut self, name: &str) -> Option<&mut T>
                 where
@@ -291,6 +330,8 @@ macro_rules! create_serde_registry_for_trait {
                     }
                 }
 
+                /// Get an element of a given type contained in this map by [`TypeId`], as mut.
+                #[must_use]
                 #[inline]
                 pub fn by_typeid_mut(
                     &mut self,
@@ -305,6 +346,8 @@ macro_rules! create_serde_registry_for_trait {
                     }
                 }
 
+                /// Get all elements of a type contained in this map.
+                #[must_use]
                 #[inline]
                 pub fn get_all<T>(
                     &self,
@@ -325,6 +368,8 @@ macro_rules! create_serde_registry_for_trait {
                     }
                 }
 
+                /// Get all elements of a given type contained in this map by [`TypeId`].
+                #[must_use]
                 #[inline]
                 pub fn all_by_typeid(
                     &self,
@@ -341,6 +386,7 @@ macro_rules! create_serde_registry_for_trait {
                     }
                 }
 
+                /// Get all elements contained in this map, as mut.
                 #[inline]
                 pub fn get_all_mut<T>(
                     &mut self,
@@ -362,6 +408,7 @@ macro_rules! create_serde_registry_for_trait {
                     }
                 }
 
+                /// Get all [`TypeId`]`s` contained in this map, as mut.
                 #[inline]
                 pub fn all_by_typeid_mut(
                     &mut self,
@@ -378,6 +425,7 @@ macro_rules! create_serde_registry_for_trait {
                     }
                 }
 
+                /// Get all [`TypeId`]`s` contained in this map.
                 #[inline]
                 pub fn all_typeids(
                     &self,
@@ -388,6 +436,7 @@ macro_rules! create_serde_registry_for_trait {
                     self.map.keys().map(|x| pack_type_id(*x))
                 }
 
+                /// Run `func` for each element in this map.
                 #[inline]
                 pub fn for_each(
                     &self,
@@ -401,6 +450,7 @@ macro_rules! create_serde_registry_for_trait {
                     Ok(())
                 }
 
+                /// Run `func` for each element in this map, getting a mutable borrow.
                 #[inline]
                 pub fn for_each_mut(
                     &mut self,
@@ -414,6 +464,7 @@ macro_rules! create_serde_registry_for_trait {
                     Ok(())
                 }
 
+                /// Insert an element into this map.
                 #[inline]
                 pub fn insert(&mut self, val: Box<dyn $trait_name>, name: &str) {
                     let id = unpack_type_id((*val).type_id());
@@ -426,11 +477,15 @@ macro_rules! create_serde_registry_for_trait {
                         .insert(xxhash_rust::xxh3::xxh3_64(name.as_bytes()), val);
                 }
 
+                /// Returns the `len` of this map.
+                #[must_use]
                 #[inline]
                 pub fn len(&self) -> usize {
                     self.map.len()
                 }
 
+                /// Returns if the element with a given type is contained in this map.
+                #[must_use]
                 #[inline]
                 pub fn contains_type<T>(&self) -> bool
                 where
@@ -439,6 +494,8 @@ macro_rules! create_serde_registry_for_trait {
                     self.map.contains_key(&unpack_type_id(TypeId::of::<T>()))
                 }
 
+                /// Returns if the element by a given `name` is contained in this map.
+                #[must_use]
                 #[inline]
                 pub fn contains<T>(&self, name: &str) -> bool
                 where
@@ -450,6 +507,8 @@ macro_rules! create_serde_registry_for_trait {
                     }
                 }
 
+                /// Create a new `SerdeAny` map.
+                #[must_use]
                 pub fn new() -> Self {
                     Self {
                         map: HashMap::default(),
@@ -493,6 +552,7 @@ macro_rules! create_serde_registry_for_trait {
 create_serde_registry_for_trait!(serdeany_registry, crate::bolts::serdeany::SerdeAny);
 pub use serdeany_registry::*;
 
+/// Implement a [`SerdeAny`], registering it in the [`RegistryBuilder`]
 #[cfg(feature = "std")]
 #[macro_export]
 macro_rules! impl_serdeany {
