@@ -7,7 +7,7 @@ use crate::{
     inputs::Input,
     observers::ObserversTuple,
     stages::StagesTuple,
-    state::HasExecutions,
+    state::{HasClientPerfStats, HasExecutions},
     utils::current_time,
     Error,
 };
@@ -15,8 +15,8 @@ use crate::{
 use alloc::string::ToString;
 use core::{marker::PhantomData, time::Duration};
 
-/// Send a stats update all 6 (or more) seconds
-const STATS_TIMEOUT_DEFAULT: Duration = Duration::from_millis(6 * 1000);
+/// Send a stats update all 3 (or more) seconds
+const STATS_TIMEOUT_DEFAULT: Duration = Duration::from_millis(3 * 1000);
 
 /// Holds a set of stages
 pub trait HasStages<CS, E, EM, I, S, ST>
@@ -122,7 +122,7 @@ pub trait Fuzzer<E, EM, S, CS> {
 
     /// Given the last time, if `stats_timeout` seconds passed, send off an info/stats/heartbeat message to the broker.
     /// Returns the new `last` time (so the old one, unless `stats_timeout` time has passed and stats have been sent)
-    /// Will return an Error, if the stats could not be sent.
+    /// Will return an [`crate::Error`], if the stats could not be sent.
     fn maybe_report_stats(
         state: &mut S,
         manager: &mut EM,
@@ -184,7 +184,7 @@ where
 impl<CS, ST, E, EM, I, OT, S> Fuzzer<E, EM, S, CS> for StdFuzzer<CS, ST, E, EM, I, OT, S>
 where
     CS: CorpusScheduler<I, S>,
-    S: HasExecutions,
+    S: HasExecutions + HasClientPerfStats,
     ST: StagesTuple<CS, E, EM, I, S>,
     EM: EventManager<I, S>,
     E: Executor<I> + HasObservers<OT>,
@@ -200,7 +200,8 @@ where
     ) -> Result<Duration, Error> {
         let cur = current_time();
         if cur - last > stats_timeout {
-            //println!("Fire {:?} {:?} {:?}", cur, last, stats_timeout);
+            // Default no introspection implmentation
+            #[cfg(not(feature = "introspection"))]
             manager.fire(
                 state,
                 Event::UpdateStats {
@@ -209,6 +210,27 @@ where
                     phantom: PhantomData,
                 },
             )?;
+
+            // If performance stats are requested, fire the `UpdatePerfStats` event
+            #[cfg(feature = "introspection")]
+            {
+                state
+                    .introspection_stats_mut()
+                    .set_current_time(crate::cpu::read_time_counter());
+
+                // Send the current stats over to the manager. This `.clone` shouldn't be
+                // costly as `ClientPerfStats` impls `Copy` since it only contains `u64`s
+                manager.fire(
+                    state,
+                    Event::UpdatePerfStats {
+                        executions: *state.executions(),
+                        time: cur,
+                        introspection_stats: Box::new(*state.introspection_stats()),
+                        phantom: PhantomData,
+                    },
+                )?;
+            }
+
             Ok(cur)
         } else {
             if cur.as_millis() % 1000 == 0 {}
@@ -223,12 +245,36 @@ where
         manager: &mut EM,
         scheduler: &CS,
     ) -> Result<usize, Error> {
+        // Init timer for scheduler
+        #[cfg(feature = "introspection")]
+        state.introspection_stats_mut().start_timer();
+
+        // Get the next index from the scheduler
         let idx = scheduler.next(state)?;
 
+        // Mark the elapsed time for the scheduler
+        #[cfg(feature = "introspection")]
+        state.introspection_stats_mut().mark_scheduler_time();
+
+        // Mark the elapsed time for the scheduler
+        #[cfg(feature = "introspection")]
+        state.introspection_stats_mut().reset_stage_index();
+
+        // Execute all stages
         self.stages_mut()
             .perform_all(state, executor, manager, scheduler, idx)?;
 
+        // Init timer for manager
+        #[cfg(feature = "introspection")]
+        state.introspection_stats_mut().start_timer();
+
+        // Execute the manager
         manager.process(state, executor, scheduler)?;
+
+        // Mark the elapsed time for the manager
+        #[cfg(feature = "introspection")]
+        state.introspection_stats_mut().mark_manager_time();
+
         Ok(idx)
     }
 }
